------------------------------------------------------
-- Gemologist.lua
------------------------------------------------------

FGO_INDENT = "  ";

-- Configuration
FGO_Config = { };
FGO_Config.Tooltip = true;
FGO_Config.ShowCost = false;

FGO_TooltipHooks = {};
function FGO_GameTooltip_OnHide()
	local tooltipName = this:GetName();
	FGO_SetTooltipHooks(tooltipName);
end

function FGO_GameTooltip_SetDefaultAnchor(tooltip, parent)
	local tooltipName = tooltip:GetName();
	FGO_SetTooltipHooks(tooltipName);
end

function FGO_SetTooltipHooks(tooltipName)
	if (not tooltipName or tooltipName == "") then return; end
	local tooltip = getglobal(tooltipName);
	if (tooltip and tooltip:HasScript("OnTooltipSetItem") and not FGO_TooltipHooks[tooltipName]) then
		FGO_TooltipHooks[tooltipName] = {};
		FGO_TooltipHooks[tooltipName].OnTooltipSetItem = tooltip:GetScript("OnTooltipSetItem");
		tooltip:SetScript("OnTooltipSetItem", FGO_OnTooltipSetItem);
	end
end

function FGO_OnTooltipSetItem()
	local tooltipName = this:GetName();
	local tooltip = getglobal(tooltipName);
	if (FGO_TooltipHooks[tooltipName] and FGO_TooltipHooks[tooltipName].OnTooltipSetItem) then
		FGO_TooltipHooks[tooltipName].OnTooltipSetItem(tooltip);
	end
	local name, link = this:GetItem();

	if ( FGO_Config.Tooltip and link ~= nil) then
		local _, _, itemID  = string.find(link, "item:(%d+)");
		itemID = tonumber(itemID);
		local itemInfo = FGO_ItemInfo[itemID];

		if (itemInfo ~= nil) then
			local color = GFW_FONT_COLOR;
			local localizedSources = {};
			for _, source in ipairs(itemInfo) do
				table.insert(localizedSources, getglobal(source));
			end
			if (table.getn(localizedSources) == 0) then
				GFWUtils.DebugLog(link.." listed but missing sources.");
				return false;
			elseif (table.getn(localizedSources) == 1) then
				this:AddLine(string.format(FGO_FOUND_ONLY_IN_FORMAT, localizedSources[1]), color.r, color.g, color.b);
				return true;
			else
				this:AddLine(FGO_FOUND_IN_HEADER, color.r, color.g, color.b);
				this:AddLine(table.concat(localizedSources, ", "), color.r, color.g, color.b);
				return true;
			end
		end
	end
end

function FGO_OnLoad()

	-- Register Slash Commands
	SLASH_GFW_GEMOLOGIST1 = "/gemologist";
	SLASH_GFW_GEMOLOGIST2 = "/gems";
	SlashCmdList["GFW_GEMOLOGIST"] = function(msg)
		FGO_ChatCommandHandler(msg);
	end
	
	hooksecurefunc("GameTooltip_OnHide", FGO_GameTooltip_OnHide);
	hooksecurefunc("GameTooltip_SetDefaultAnchor", FGO_GameTooltip_SetDefaultAnchor);
	FGO_SetTooltipHooks("GameTooltip");
	FGO_SetTooltipHooks("ItemRefTooltip");
	FGO_SetTooltipHooks("ShoppingTooltip1");
	FGO_SetTooltipHooks("ShoppingTooltip2");
	
	local version = GetAddOnMetadata("GFW_Gemologist", "Version");
	GFWUtils.Print("Fizzwidget Gemologist "..version.." initialized!");
end

function FGO_ChatCommandHandler(msg)

	-- Print Help
	if ( msg == "help" ) or ( msg == "" ) then
		local version = GetAddOnMetadata("GFW_Gemologist", "Version");
		GFWUtils.Print("Fizzwidget Gemologist "..version..":");
		GFWUtils.Print("/gemologist (or /gems)");
		GFWUtils.Print("- "..GFWUtils.Hilite("help").." - Print this helplist.");
		GFWUtils.Print("- "..GFWUtils.Hilite("status").." - Check current settings.");
		GFWUtils.Print("- "..GFWUtils.Hilite("tooltip on").." | "..GFWUtils.Hilite("off").." - Enable/disable display of mining info in gem tooltips.");
		GFWUtils.Print("- "..GFWUtils.Hilite("info <item link>").." - Show mining info for a gem in the chat window.");
		return;
	end
	
	if (msg == "version") then
		local version = GetAddOnMetadata("GFW_Gemologist", "Version");
		GFWUtils.Print("Fizzwidget Gemologist "..version);
		return;
	end
	
	if (msg == "tooltip on") then
		FGO_Config.Tooltip = true;
		GFWUtils.Print("Showing mining information in tooltips for gems.");
		return;
	end
	if (msg == "tooltip off") then
		FGO_Config.Tooltip = false;
		GFWUtils.Print(GFWUtils.Hilite("Not").." showing mining information in tooltips for gems.");
		return;
	end
	
	
	if ( msg == "status" ) then
		if ( FGO_Config.Tooltip ) then
			GFWUtils.Print("Showing mining information in tooltips for gems.");
		else
			GFWUtils.Print(GFWUtils.Hilite("Not").." showing mining information in tooltips for gems.");
		end
		return;
	end
	
	local _, _, cmd, itemLink = string.find(msg, "(%w+) (.+)");
	if (cmd == "info") then
		local _, _, itemID  = string.find(itemLink, "item:(%d+)");
		if (itemID == nil or itemID == "") then
			GFWUtils.Print("Usage: "..GFWUtils.Hilite("/gem info <item link>"));
			return;
		end
		itemID = tonumber(itemID);
		
		local itemInfo = FGO_ItemInfo[itemID];
		if (itemInfo ~= nil) then
			local localizedSources = {};
			for _, source in ipairs(itemInfo) do
				table.insert(localizedSources, getglobal(source));
			end
			if (table.getn(localizedSources) == 0) then
				GFWUtils.DebugLog(itemLink.." is listed but missing sources.");
				return false;
			elseif (table.getn(localizedSources) == 1) then
				GFWUtils.Print(string.format(FGO_LINK_FOUND_ONLY_IN_FORMAT, itemLink, localizedSources[1]));
				return true;
			else
				GFWUtils.Print(string.format(FGO_LINK_FOUND_IN_HEADER_FORMAT, itemLink));
				GFWUtils.Print(table.concat(localizedSources, ", "));
				return true;
			end
		end

		GFWUtils.Print("Nothing known about "..itemLink..".");
		return;
	end
	
	-- if we made it down here, there were args we didn't understand... time to remind the user what to do.
	FGO_ChatCommandHandler("help");

end

------------------------------------------------------
-- Runtime loading
------------------------------------------------------

FGO_OnLoad();